##############################################################################
# Copyright (C) 2001 Kim Nikolay <fafhrd@datacom.kz>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################
"""Customizable MessageCatalog objects that come from the filesystem."""
__version__='$Revision: 0.1 $'[11:-2]

import string, os

import Globals
from DateTime import DateTime
from AccessControl import ClassSecurityInfo

from utils import _dtmldir
from Products.CMFCore.CMFCorePermissions import ViewManagementScreens, View
from Products.CMFCore.FSObject import FSObject
from Products.CMFCore.DirectoryView \
     import registerFileExtension, registerMetaType, expandpath

from Products.Localizer.MessageCatalog import MessageCatalog

class FSMessageCatalog(FSObject, MessageCatalog):
    """FSMessageCatalog."""

    meta_type = 'Filesystem MessageCatalog'

    def manage_options(self):
        """ """
        o = [{'label':'Customize', 'action':'manage_main'}]
        o += MessageCatalog.manage_options(self)
        return o
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    def __init__(self, id, filepath, fullname=None, properties=None):
        FSObject.__init__(self, id, filepath, fullname, properties)

    security.declareProtected(ViewManagementScreens, 'manage_main')
    manage_main = Globals.DTMLFile('cust_messages', _dtmldir)

    def _createZODBClone(self):
        langs = self.get_languages()
        msg = MessageCatalog(self.id, self.title, langs)

        # wrong way
        msg.manage_changeDefaultLang(self._default_language)
        
        for lang in langs:
            msg.manage_import(lang, self.manage_export(lang))
        return msg

    def _readFile(self, reparse):
        fp = expandpath(self._filepath)
        file = open(fp, 'rb')
        try: data = file.readlines()
        finally: file.close()

        rawparams = []
        try: rawparams = [line[2:].strip() for line in data]
        except: pass

        params = {}
        map(lambda s, d=params: d.setdefault(*s.split('=', 1)), rawparams)

        # get languages
        langs = [ln for ln in params.get('languages','en').strip().split(' ')]

        # init MessageCatalog
        MessageCatalog.__init__(self, self.id, params.get('title', ''), langs)

        # set default lang
        self.manage_changeDefaultLang(params.get('default', langs[0]))
        
        # load messages from po
        lp = os.path.normpath("%s/%s/" % (os.path.dirname(fp),
                                          params.get('locale', 'locale')))
        for lang in langs:
            try:
                pofile = open("%s/%s.po" % (lp, lang), 'rb')
                self.manage_import(lang, pofile)
            except IOError:
                pass

    security.declarePublic('gettext')
    def gettext(self, message, lang=None, add=0):
        """Returns the message translation from the database if available."""
        return MessageCatalog.gettext(self, message, lang, 0)
    
    __call__ = gettext


Globals.InitializeClass(FSMessageCatalog)

registerFileExtension('msg', FSMessageCatalog)
registerMetaType('MessageCatalog', FSMessageCatalog)
