##############################################################################
# Copyright (C) 2001 Kim Nikolay <fafhrd@datacom.kz>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################

"""Localiazer gettext Expression Engine

implementation of TALES, with handlers for gettext.
"""
__version__='$Revision: 0.2 $'[11:-2]

import sys, types
from Products.PageTemplates import Expressions

class GettextExpr:
    """ gettext expression by Localizer
    like <b tal:content="gettext:lang">Text</b>
    
    """

    def __init__(self, name, expr, engine):
        self._name = name
        self._expr = expr.strip()

    def __call__(self, econtext):
        lo = econtext.local_vars
        gl_get = econtext.global_vars.get
        lo_has_key = econtext.local_vars.has_key

        lang = None
        msgCatalog = None
        message = econtext.tal_tagContent[0][1][0]

        if self._expr:
            val = econtext.evaluate(self._expr)
            if type(val) == types.StringType:
                message = val
            else:
                if type(val) == types.DictType:
                    lang = val.get('lang', None)
                    message = val.get('msg', message)
                    msgCatalog = val.get('msgCatalog', None)

        # dstLang
        if not lang:
            if lo_has_key('lang'):
                lang = lo['lang']
            else:
                lang = gl_get('lang', None)

        add = 1

        try:
            # get MessageCatalog object id
            if not msgCatalog:
                if lo_has_key('messageCatalog'):
                    msgCatalog = lo['messageCatalog']
                else:
                    msgCatalog = gl_get(
                        'messageCatalog',
                        getattr(econtext.contexts['here'], 'Messages'))
            return msgCatalog(message, lang, add)
        except:
            return message

    def __str__(self):
        return '%s expression %s' % (self._name, `self._expr`)

    def __repr__(self):
        return '%s:%s' % (self._name, `self._expr`)


def initialize():
    engine = Expressions.getEngine()
    if not engine.getTypes().has_key('gettext'):
        engine.registerType('gettext', GettextExpr)
